/**
 * This is the JavaScript portion of the Magic Browser Client.
 * Copyright 2000 Magic Software Enterprises Ltd. All rights reserved (c).
 * MGBrowserClient.js version 9.4
 */

/************************************************************************/
/* Global Variables                                                     */
/************************************************************************/
var JS_TYPE_UNDEF       = "undefined";
var EMPTY_STR           = "";
var HTML_COMMA          = "&#044;"           // has to be the same as ConstInterface.HTML_COMMA
var COMMA_STR           = ",";
var EXTERNAL            = "external";


//these 4 var should not be removed since they are updated via c# and if they will not exist in the js it
//wi throw an exception in c#
var mainWnd;              // reference to main window in every magic window
var numberWnd;                               // current window number
var number;                 
var tsk;

var newWinRef           = null;


/**
 * a wrapper for the old callProcessEvent that was converted to c#
 */
function callProcessEvent(type, name, value, srcNum, tsk, winNum, modifier, keyCode,numWnd)
{
    type = (type == null ? "" : type);     
    name = (name == null ? "" : name);     
    value = (value == null ? "" : value);     
    srcNum = (srcNum == null ? "" : srcNum);     
    tsk = (tsk == null ? "" : tsk);     
    winNum = (winNum == null ? "" : winNum);    
    modifier = (modifier == null ? "" : modifier);
    keyCode = (keyCode == null ? "" : keyCode);
      
    var appl = getApplet();
    if ( appl != null)
        try
        {
           appl.callProcessEvent(type, name, value, srcNum, tsk, winNum, modifier, keyCode,numWnd);
        }catch(e){}
          
}

/**
 * display or hide all the contents of a sub tree of a DOM object
 * called by displayRow()
 * @param obj reference to row or a descendent object of it
 * @param boolean if the row need be dysplayed
 */
function changeSubtreeVisibility(obj, display)
{
   var visibility = display ? "" : "hidden";
   var childNum;

   obj.style.visibility = visibility;
   // scan the child objects
   if ((obj.children != null && typeof obj.children == "object") && obj.children.length > 0)
   {
      for (childNum = 0; childNum < obj.children.length; childNum++)
          changeSubtreeVisibility(obj.children[childNum], display);
   }
}


/**
 * This function should always be called on main window
 * Execute a "timer" event
 * @param seconds the time interval in seconds
 * @param isIdleTimer true if this is the idle time "timer" event
 * @param numWnd the number of the window on which the timer is to be executed
 */
function runTimer(seconds, isIdleTimer, numWnd)
{
    var appl = getApplet();
    if ( appl != null)
    {
        try
        {
           appl.runTimer(seconds, isIdleTimer, numWnd);
        }catch(e) {}
    }
}

/**
 * add event processing to non Magic controls
 * @param window reference in form of string
 */
function addProcessingToNonMagicCtrls()
{
   var win = self;
   var elements = win.document.all;
   var elem;
   var i;
   var isTable;
   var realType;
   var mWnd = mainWnd;
   
   for (i = 0; i < elements.length; i++)
   {
      elem = elements[i];
              
      realType = getCtrlType(elem);
      
      if (realType != null  && !isMagicCtrl(elem))
      {
         isTable = (typeof elem.tagName != "undefined" && elem.tagName == "TABLE");
         // add handling only if there is no user defined handler for the "focus" event
         if (!isTable && elem.onfocus == null)
            elem.onfocus = MGEventProcessing;
      }
   }
}

/**
 *Is the control was initialized at parsing time, like Magic control
 */
function isMagicCtrl(ctrl)
{
   if (typeof ctrl.number != JS_TYPE_UNDEF)
      return true;
   if (typeof ctrl.length == "number" && ctrl.length > 0)
      return isMagicCtrl(ctrl[0]);
   return false;
} 
 
/**
 * @return type of control, or null if the control not identified
 */
function getCtrlType(ctrl)
{
   var realType = null;
   var type;

   if (!(ctrl != null && typeof ctrl == "object"))
      return null;

   if (typeof ctrl.length == "number")
   {
      if ((ctrl[0] != null && typeof ctrl[0] == "object") &&
           typeof ctrl[0].type   != JS_TYPE_UNDEF &&
           typeof ctrl.tagName   == JS_TYPE_UNDEF &&
           ctrl[0].tagName       == "INPUT")
         realType = CTRL_TYPE_RADIO;
      else if (typeof ctrl.tagName != JS_TYPE_UNDEF)
      {
       var name = ctrl.tagName;
       if (name == "SELECT" || name == "select-multiple" ||
            name == "select-one")
         realType = 'S';
       }
   }
   else
   {
      if (typeof ctrl.type != JS_TYPE_UNDEF)
      {
         type = ctrl.type.toUpperCase();
         if (type == "BUTTON")
           realType = 'B';
         else if (type == "CHECKBOX")
            realType = 'C';
         else if (type == "TEXT" || type == "TEXTAREA" || type == "PASSWORD" ||
                   (ctrl.tagName == "INPUT" && type == ""))
             realType = 'T';
         else if (type == "IMG" || type == "IMAGE")
             realType = 'I';
         else if (typeof ctrl.tagName != JS_TYPE_UNDEF && ctrl.tagName == "A")
            realType = 'H';
       }
       else if (typeof ctrl.tagName != JS_TYPE_UNDEF)
       {
          type = ctrl.tagName;
          if (type == "TABLE")
            realType = 'A';
          else if (type == "IMG" || type == "IMAGE")
             realType = 'I';
          else if (type == "A" || type == "DIV")
             realType = 'H';
          else if (type == "IFRAME")
            realType = 'M';
       }
   }
 return realType;
}

/**
 * returns a reference to the main Applet should be called only from the mainWnd
 */
function getApplet()
{
    return self.document.all["MGBC_ASSEMBLY"];
}

/**
 * Process Magic External Event
 */
function MGExternalEvent()
{
   var args    = MGExternalEvent.arguments;
   var obj;
   var re      = /,/g;
   var length  = args.length;
   var value   = EMPTY_STR;
   var i       = 0;

   // build a value string from the arguments
   while (i < length)
   {
      obj  = EMPTY_STR + args[i];
      obj  = obj.replace(re, HTML_COMMA); // change inner commas to HTML_COMMA
      value += obj;
      if (++i < length) // don't add a comma after the last block
         value += COMMA_STR;
   }
   mainWnd.callProcessEvent(EXTERNAL, null, value, length, null, self.numberWnd, null, null,numberWnd);
}

 function wrapperLogit(msg)
 {
    var app = getApplet();
    if ( app != null || typeof app != JS_TYPE_UNDEF)
        app.logit(msg);
 }
  
 /**
  * since we have a problem call user defined function from c# we use this function which has a constant signature
  */
 function callJs(cmd)
 {
   var ret = null;
   
   try
   {
      ret = eval(cmd);
   }
   catch (e)
   {
      // if the function to be called does not exist or 
      // if the command to be evaluated fails, 
      // simply ignore.
   }
   
   return ret;
 }
 
 /**
  * this function is call by the batch redirect intermidate html
  * does the logic needed in order to call browser client window from batch
  * this function is called from the inner IFRAM on the main window only
  * in order to allow c# Compatibility - the same function exists in java version 
  */
  function callBatchOpenWin(t,u)
  {
  
    if ( numberWnd != 0)
    {
        alert("callBatchOpenWin should be called only on the main window");
        return;
    }
  
    var appl = getApplet();
    
    if ( appl != null)
        appl.callBatchOpenWin(t,u);
  }
  
 
 /*---------------------------------------------------------------------------------------------------------
  *
  *                     THIRD STAGE JS FUNCTIONS - DO NOT CONVERT THIS FUNCTIONS
  *
  *----------------------------------------------------------------------------------------------------------
  */
  
   var parentWnd = null; //THIS IS THE MAIN WINDOW ON WHICH THE ASSEMBLY IS LOCATED
                         //we need it because we can not activate method in the assembly directrly from other
                         //window that was also the reason we needed wrappers in stage 2
   var loadHandled = false; // since we move the location of trapping events on new windows ( now its before load event)
                            // we must make sure that otrher event will be dicarded if the load event did not arrived yet 
                            //relevat only for child window since in the maun window we still trapp events in the load eevent handlling
                         
   
   //this function should be called by the events trapped in js see remark line is web_info fro example
   function MGEventProcessing(evt)
   {
      if ( self.document.all["MGBC_ASSEMBLY"] == null && evt != null && evt.type == "load")
      {
            loadHandled = true;
            return;
      }
     
        
      parentWnd = mainWnd;
      
      if ( parentWnd == null && self.document.all["MGBC_ASSEMBLY"] != null)
      {
        loadHandled = true;
        parentWnd = self;
      }
      
      //discard events befor load event on child windows
      if (!loadHandled)
          return;    
  
      //a delegator that moves the event to the conataining window (main window) and from there to 
      //the method in the assembly the handles the events
      try
      {         
        if ( parentWnd != null && typeof parentWnd['catchEvent'] != JS_TYPE_UNDEF)
	        return parentWnd.catchEvent(evt,self);
	  }
	  catch(e)
	  {
	    //alert("in catch");
	    return;
	  }
   }
   
function catchEvent(evt,win)
{

    if ( evt == null)
    {
        evt = win.event;
        if ( evt == null)
        {
            //alert( "no event found returning");
            return;
        }
    }
    // in cases were the the source element of the evnt is null we put our own attribute
    // on the event with the window in it since without the window (in c#) we can do nothing 
    // we need to put a new attribute since events attributes are read only
	if ( evt.srcElement == null || typeof evt.srcElement == JS_TYPE_UNDEF)
	    evt.mySrc = win;
	    
	var app = getApplet();
    if ( app != null || typeof app != JS_TYPE_UNDEF)
	    return app.MGEventProcessing(evt);
	else
	{
	    //alert("applet was not found returning");
	    return;
	}
}

function newWin(windowFeatures)
{
  mainWnd.newWinRef = open("", "", windowFeatures);
  return mainWnd.newWinRef;
}
